@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2018, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -20,8 +20,11 @@ from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-
 from config import PG_DEFAULT_DRIVER
+from pgadmin.utils import IS_PY2
+# If we are in Python3
+if not IS_PY2:
+    unicode = str
 
 
 class FtsTemplateModule(SchemaChildModule):
@@ -52,6 +55,7 @@ class FtsTemplateModule(SchemaChildModule):
         self.min_ver = None
         self.max_ver = None
         super(FtsTemplateModule, self).__init__(*args, **kwargs)
+        self.min_gpdbver = 1000000000
 
     def get_nodes(self, gid, sid, did, scid):
         """
@@ -291,7 +295,9 @@ class FtsTemplateView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(gettext("""Could not find the FTS template node in the database."""))
+            return gone(
+                gettext("Could not find the requested FTS template.")
+            )
 
         return ajax_response(
             response=res['rows'][0],
@@ -385,6 +391,9 @@ class FtsTemplateView(PGChildNodeView):
 
         # Fetch sql query to update fts template
         sql, name = self.get_sql(gid, sid, did, scid, data, tid)
+        # Most probably this is due to error
+        if not isinstance(sql, (str, unicode)):
+            return sql
         sql = sql.strip('\n').strip(' ')
         status, res = self.conn.execute_scalar(sql)
         if not status:
@@ -483,6 +492,10 @@ class FtsTemplateView(PGChildNodeView):
         # Fetch sql query for modified data
         # Fetch sql query for modified data
         SQL, name = self.get_sql(gid, sid, did, scid, data, tid)
+        # Most probably this is due to error
+        if not isinstance(SQL, (str, unicode)):
+            return SQL
+
         if SQL == '':
             SQL = "--modified SQL"
 
@@ -512,6 +525,10 @@ class FtsTemplateView(PGChildNodeView):
             status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
+            if len(res['rows']) == 0:
+                return gone(
+                    gettext("Could not find the requested FTS template.")
+                )
 
             old_data = res['rows'][0]
 
