@@ -1,15 +1,16 @@
 define('pgadmin.node.server', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'pgadmin', 'pgadmin.browser', 'alertify',
+  'underscore.string', 'sources/pgadmin', 'pgadmin.browser',
   'pgadmin.server.supported_servers',
   'pgadmin.user_management.current_user',
-  'sources/alerts/alertify_wrapper',
+  'pgadmin.alertifyjs', 'pgadmin.browser.server.privilege'
 ], function(
-  gettext, url_for, $, _, S, pgAdmin, pgBrowser, alertify,
-  supported_servers, current_user, AlertifyWrapper
+  gettext, url_for, $, _, S, pgAdmin, pgBrowser,
+  supported_servers, current_user, alertify
 ) {
 
   if (!pgBrowser.Nodes['server']) {
+    var SSL_MODES = ['prefer', 'require', 'verify-ca', 'verify-full'];
 
     var SecurityModel = pgBrowser.SecLabelModel = pgBrowser.Node.Model.extend({
       defaults: {
@@ -23,6 +24,7 @@ define('pgadmin.node.server', [
       },{
         id: 'label', label: gettext('Security Label'),
         type: 'text', editable: true,
+        cellHeaderClasses:'override_label_class_font_size'
       }],
       validate: function() {
         var err = {},
@@ -42,7 +44,7 @@ define('pgadmin.node.server', [
     });
 
     pgAdmin.Browser.Nodes['server'] = pgAdmin.Browser.Node.extend({
-      parent_type: 'server-group',
+      parent_type: 'server_group',
       type: 'server',
       dialogHelp: url_for('help.static', {'filename': 'server_dialog.html'}),
       label: gettext('Server'),
@@ -61,7 +63,7 @@ define('pgadmin.node.server', [
         this.initialized = true;
 
         pgBrowser.add_menus([{
-          name: 'create_server_on_sg', node: 'server-group', module: this,
+          name: 'create_server_on_sg', node: 'server_group', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
           category: 'create', priority: 1, label: gettext('Server...'),
           data: {action: 'create'}, icon: 'wcTabIcon icon-server'
@@ -189,8 +191,7 @@ define('pgadmin.node.server', [
               type:'DELETE',
               success: function(res) {
                 if (res.success == 1) {
-                  var alertifyWrapper = new AlertifyWrapper();
-                  alertifyWrapper.success(res.info);
+                  alertify.success(res.info);
                   d = t.itemData(i);
                   t.removeIcon(i);
                   d.connected = false;
@@ -209,8 +210,7 @@ define('pgadmin.node.server', [
                 }
                 else {
                   try {
-                    var alertifyWrapper = new AlertifyWrapper();
-                    alertifyWrapper.error(res.errormsg);
+                    alertify.error(res.errormsg);
                   } catch (e) {}
                   t.unload(i);
                 }
@@ -219,8 +219,7 @@ define('pgadmin.node.server', [
                 try {
                   var err = $.parseJSON(xhr.responseText);
                   if (err.success == 0) {
-                    var alertifyWrapper = new AlertifyWrapper();
-                    alertifyWrapper.error(err.errormsg);
+                    alertify.error(err.errormsg);
                   }
                 } catch (e) {}
                 t.unload(i);
@@ -288,20 +287,17 @@ define('pgadmin.node.server', [
                 method:'GET',
                 success: function(res) {
                   if (res.data.status) {
-                    var alertifyWrapper = new AlertifyWrapper();
-                    alertifyWrapper.success(res.data.result);
+                    alertify.success(res.data.result);
                   }
                   else {
-                    var alertifyWrapper = new AlertifyWrapper();
-                    alertifyWrapper.error(res.data.result);
+                    alertify.error(res.data.result);
                   }
                 },
                 error: function(xhr, status, error) {
                   try {
                     var err = $.parseJSON(xhr.responseText);
                     if (err.success == 0) {
-                      var alertifyWrapper = new AlertifyWrapper();
-                      alertifyWrapper.error(err.errormsg);
+                      alertify.error(err.errormsg);
                     }
                   } catch (e) {}
                   t.unload(i);
@@ -336,15 +332,13 @@ define('pgadmin.node.server', [
                 method:'POST',
                 data:{ 'value': JSON.stringify(value) },
                 success: function(res) {
-                  var alertifyWrapper = new AlertifyWrapper();
-                  alertifyWrapper.success(res.data.result, 10);
+                  alertify.success(res.data.result, 10);
                 },
                 error: function(xhr, status, error) {
                   try {
                     var err = $.parseJSON(xhr.responseText);
                     if (err.success == 0) {
-                      var alertifyWrapper = new AlertifyWrapper();
-                      alertifyWrapper.error(err.errormsg, 10);
+                      alertify.error(err.errormsg, 10);
                     }
                   } catch (e) {}
                   t.unload(i);
@@ -352,8 +346,7 @@ define('pgadmin.node.server', [
               });
              } else {
                 evt.cancel = true;
-                var alertifyWrapper = new AlertifyWrapper();
-                alertifyWrapper.error( gettext('Please enter a valid name.'), 10);
+                alertify.error( gettext('Please enter a valid name.'), 10);
              }
            },
            // We will execute this function when user clicks on the Cancel button
@@ -372,7 +365,9 @@ define('pgadmin.node.server', [
             i = input.item || t.selected(),
             d = i && i.length == 1 ? t.itemData(i) : undefined,
             node = d && pgBrowser.Nodes[d._type],
-            url = obj.generate_url(i, 'change_password', d, true);
+            url = obj.generate_url(i, 'change_password', d, true),
+            is_pgpass_file_used = false,
+            check_pgpass_url = obj.generate_url(i, 'check_pgpass', d, true);
 
           if (!d)
             return false;
@@ -394,8 +389,8 @@ define('pgadmin.node.server', [
                   type: 'text', disabled: true, control: 'input'
                 },{
                   name: 'password', label: gettext('Current Password'),
-                  type: 'password', disabled: false, control: 'input',
-                  required: true
+                  type: 'password', disabled: function() { return is_pgpass_file_used },
+                  control: 'input', required: true
                 },{
                   name: 'newPassword', label: gettext('New Password'),
                   type: 'password', disabled: false, control: 'input',
@@ -417,9 +412,11 @@ define('pgadmin.node.server', [
                  setup:function() {
                   return {
                     buttons: [{
-                      text: gettext('Ok'), key: 13, className: 'btn btn-primary', attrs:{name:'submit'}
-                      },{
-                      text: gettext('Cancel'), key: 27, className: 'btn btn-danger', attrs:{name:'cancel'}
+                      text: gettext('Ok'), key: 13, className: 'btn btn-primary',
+                      attrs:{name:'submit'}
+                    },{
+                      text: gettext('Cancel'), key: 27, className: 'btn btn-danger',
+                      attrs:{name:'cancel'}
                     }],
                     // Set options for dialog
                     options: {
@@ -443,15 +440,18 @@ define('pgadmin.node.server', [
                 },
                 prepare: function() {
                   var self = this;
-                  // Disable Backup button until user provides Filename
+                  // Disable Ok button until user provides input
                   this.__internal.buttons[0].element.disabled = true;
-                  var $container = $("<div class='change_password'></div>"),
-                    newpasswordmodel = new newPasswordModel({'user_name': self.user_name});
 
-                  var view = this.view = new Backform.Form({
-                    el: $container,
-                    model: newpasswordmodel,
-                    fields: passwordChangeFields});
+                  var $container = $("<div class='change_password'></div>"),
+                    newpasswordmodel = new newPasswordModel(
+                      {'user_name': self.user_name}
+                    ),
+                    view = this.view = new Backform.Form({
+                      el: $container,
+                      model: newpasswordmodel,
+                      fields: passwordChangeFields
+                    });
 
                   view.render();
 
@@ -464,7 +464,9 @@ define('pgadmin.node.server', [
                         newPassword = this.get('newPassword'),
                         confirmPassword = this.get('confirmPassword');
 
-                    if (_.isUndefined(password) || _.isNull(password) || password == '' ||
+                    // Only check password field if pgpass file is not available
+                    if ((!is_pgpass_file_used &&
+                         (_.isUndefined(password) || _.isNull(password) || password == '')) ||
                         _.isUndefined(newPassword) || _.isNull(newPassword) || newPassword == '' ||
                         _.isUndefined(confirmPassword) || _.isNull(confirmPassword) || confirmPassword == '') {
                       self.__internal.buttons[0].element.disabled = true;
@@ -494,20 +496,28 @@ define('pgadmin.node.server', [
                       method:'POST',
                       data:{'data': JSON.stringify(args) },
                       success: function(res) {
-                        var alertifyWrapper = new AlertifyWrapper();
                         if (res.success) {
-                          alertifyWrapper.success(res.info);
+                          // Notify user to update pgpass file
+                          if(is_pgpass_file_used) {
+                            alertify.alert(
+                              gettext("Change Password"),
+                              gettext("Please make sure to disconnect the server"
+                              + " and update the new password in the pgpass file"
+                              + " before performing any other operation")
+                            );
+                          }
+
+                          alertify.success(res.info);
                           self.close();
                         } else {
-                          alertifyWrapper.error(res.errormsg);
+                          alertify.error(res.errormsg);
                         }
                       },
                       error: function(xhr, status, error) {
                         try {
                           var err = $.parseJSON(xhr.responseText);
                           if (err.success == 0) {
-                            var alertifyWrapper = new AlertifyWrapper();
-                            alertifyWrapper.error(err.errormsg);
+                            alertify.error(err.errormsg);
                           }
                         } catch (e) {}
                       }
@@ -518,7 +528,26 @@ define('pgadmin.node.server', [
             });
           }
 
-          alertify.changeServerPassword(d).resizeTo('40%','52%');
+          // Call to check if server is using pgpass file or not
+          $.ajax({
+            url: check_pgpass_url,
+            method:'GET',
+            success: function(res) {
+              if (res.success && res.data.is_pgpass) {
+                is_pgpass_file_used = true;
+              }
+              alertify.changeServerPassword(d).resizeTo('40%','52%');
+            },
+            error: function(xhr, status, error) {
+              try {
+                var err = $.parseJSON(xhr.responseText);
+                if (err.success == 0) {
+                  alertify.error(err.errormsg);
+                }
+              } catch (e) {}
+            }
+          });
+
           return false;
         },
 
@@ -540,8 +569,7 @@ define('pgadmin.node.server', [
             dataType: "json",
             success: function(res) {
               if (res.success == 1) {
-                var alertifyWrapper = new AlertifyWrapper();
-                alertifyWrapper.success(res.info);
+                alertify.success(res.info);
                 t.itemData(i).wal_pause=res.data.wal_pause;
                 t.unload(i);
                 t.setInode(i);
@@ -557,8 +585,7 @@ define('pgadmin.node.server', [
                 var err = $.parseJSON(xhr.responseText);
                 if (err.success == 0) {
                   msg = S(err.errormsg).value();
-                  var alertifyWrapper = new AlertifyWrapper();
-                  alertifyWrapper.error(err.errormsg);
+                  alertify.error(err.errormsg);
                 }
               } catch (e) {}
               t.unload(i);
@@ -584,8 +611,7 @@ define('pgadmin.node.server', [
             dataType: "json",
             success: function(res) {
               if (res.success == 1) {
-                var alertifyWrapper = new AlertifyWrapper();
-                alertifyWrapper.success(res.info);
+                alertify.success(res.info);
                 t.itemData(i).wal_pause=res.data.wal_pause;
                 t.unload(i);
                 t.setInode(i);
@@ -601,8 +627,7 @@ define('pgadmin.node.server', [
                 var err = $.parseJSON(xhr.responseText);
                 if (err.success == 0) {
                   msg = S(err.errormsg).value();
-                  var alertifyWrapper = new AlertifyWrapper();
-                  alertifyWrapper.error(err.errormsg);
+                  alertify.error(err.errormsg);
                 }
               } catch (e) {}
               t.unload(i);
@@ -624,14 +649,21 @@ define('pgadmin.node.server', [
           role: null,
           connect_now: true,
           password: undefined,
-          save_password: false
+          save_password: false,
+          db_res: '',
+          passfile: undefined,
+          sslcompression: false,
+          sslcert: undefined,
+          sslkey: undefined,
+          sslrootcert: undefined,
+          sslcrl: undefined
         },
         // Default values!
         initialize: function(attrs, args) {
           var isNew = (_.size(attrs) === 0);
 
           if (isNew) {
-            this.set({'gid': args.node_info['server-group']._id});
+            this.set({'gid': args.node_info['server_group']._id});
           }
           pgAdmin.Browser.Node.Model.prototype.initialize.apply(this, arguments);
         },
@@ -642,7 +674,7 @@ define('pgadmin.node.server', [
           mode: ['properties', 'edit', 'create']
         },{
           id: 'gid', label: gettext('Server group'), type: 'int',
-          control: 'node-list-by-id', node: 'server-group',
+          control: 'node-list-by-id', node: 'server_group',
           mode: ['create', 'edit'], select2: {allowClear: false}
         },{
           id: 'server_type', label: gettext('Server type'), type: 'options',
@@ -651,12 +683,19 @@ define('pgadmin.node.server', [
         },{
           id: 'connected', label: gettext('Connected?'), type: 'switch',
           mode: ['properties'], group: gettext('Connection'), 'options': {
-            'onText':   'True', 'offText':  'False', 'onColor':  'success',
+            'onText':  gettext('True'), 'offText':  gettext('False'), 'onColor':  'success',
             'offColor': 'danger', 'size': 'small'
           }
         },{
           id: 'version', label: gettext('Version'), type: 'text', group: null,
           mode: ['properties'], visible: 'isConnected'
+        },{
+          id: 'bgcolor', label: gettext('Background'), type: 'color',
+          group: null, mode: ['edit', 'create'], disabled: 'isfgColorSet',
+          deps: ['fgcolor']
+        },{
+          id: 'fgcolor', label: gettext('Foreground'), type: 'color',
+          group: null, mode: ['edit', 'create'], disabled: 'isConnected',
         },{
           id: 'connect_now', controlLabel: gettext('Connect now?'), type: 'checkbox',
           group: null, mode: ['create']
@@ -666,9 +705,6 @@ define('pgadmin.node.server', [
         },{
           id: 'host', label: gettext('Host name/address'), type: 'text', group: gettext('Connection'),
           mode: ['properties', 'edit', 'create'], disabled: 'isConnected'
-        },{
-          id: 'hostaddr', label: gettext('Host address'), type: 'text', group: gettext('Advanced'),
-          mode: ['properties', 'edit', 'create'], disabled: 'isConnected'
         },{
           id: 'port', label: gettext('Port'), type: 'int', group: gettext('Connection'),
           mode: ['properties', 'edit', 'create'], disabled: 'isConnected', min: 1024, max: 65535
@@ -697,16 +733,106 @@ define('pgadmin.node.server', [
           id: 'role', label: gettext('Role'), type: 'text', group: gettext('Connection'),
           mode: ['properties', 'edit', 'create'], disabled: 'isConnected'
         },{
-          id: 'sslmode', label: gettext('SSL mode'), type: 'options', group: gettext('Connection'),
+          id: 'sslmode', label: gettext('SSL mode'), type: 'options', group: gettext('SSL'),
           mode: ['properties', 'edit', 'create'], disabled: 'isConnected',
           'options': [
-            {label: 'Allow', value: 'allow'},
-            {label: 'Prefer', value: 'prefer'},
-            {label: 'Require', value: 'require'},
-            {label: 'Disable', value: 'disable'},
-            {label: 'Verify-CA', value: 'verify-ca'},
-            {label: 'Verify-Full', value: 'verify-full'}
+            {label: gettext('Allow'), value: 'allow'},
+            {label: gettext('Prefer'), value: 'prefer'},
+            {label: gettext('Require'), value: 'require'},
+            {label: gettext('Disable'), value: 'disable'},
+            {label: gettext('Verify-CA'), value: 'verify-ca'},
+            {label: gettext('Verify-Full'), value: 'verify-full'}
           ]
+        },{
+          id: 'sslcert', label: gettext('Client certificate'), type: 'text',
+          group: gettext('SSL'), mode: ['edit', 'create'],
+          disabled: 'isSSL', control: Backform.FileControl,
+          dialog_type: 'select_file', supp_types: ['*'],
+          deps: ['sslmode']
+        },{
+          id: 'sslkey', label: gettext('Client certificate key'), type: 'text',
+          group: gettext('SSL'), mode: ['edit', 'create'],
+          disabled: 'isSSL', control: Backform.FileControl,
+          dialog_type: 'select_file', supp_types: ['*'],
+          deps: ['sslmode']
+        },{
+          id: 'sslrootcert', label: gettext('Root certificate'), type: 'text',
+          group: gettext('SSL'), mode: ['edit', 'create'],
+          disabled: 'isSSL', control: Backform.FileControl,
+          dialog_type: 'select_file', supp_types: ['*'],
+          deps: ['sslmode']
+        },{
+          id: 'sslcrl', label: gettext('Certificate revocation list'), type: 'text',
+          group: gettext('SSL'), mode: ['edit', 'create'],
+          disabled: 'isSSL', control: Backform.FileControl,
+          dialog_type: 'select_file', supp_types: ['*'],
+          deps: ['sslmode']
+        },{
+          id: 'sslcompression', label: gettext('SSL compression?'), type: 'switch',
+          mode: ['edit', 'create'], group: gettext('SSL'),
+          'options': { 'onText':   gettext('True'), 'offText':  gettext('False'),
+          'onColor':  'success', 'offColor': 'danger', 'size': 'small'},
+          deps: ['sslmode'], disabled: 'isSSL'
+        },{
+          id: 'sslcert', label: gettext('Client certificate'), type: 'text',
+          group: gettext('SSL'), mode: ['properties'],
+          deps: ['sslmode'],
+          visible: function(m) {
+            var sslcert = m.get('sslcert');
+            return !_.isUndefined(sslcert) && !_.isNull(sslcert);
+          }
+        },{
+          id: 'sslkey', label: gettext('Client certificate key'), type: 'text',
+          group: gettext('SSL'), mode: ['properties'],
+          deps: ['sslmode'],
+          visible: function(m) {
+            var sslkey = m.get('sslkey');
+            return !_.isUndefined(sslkey) && !_.isNull(sslkey);
+          }
+        },{
+          id: 'sslrootcert', label: gettext('Root certificate'), type: 'text',
+          group: gettext('SSL'), mode: ['properties'],
+          deps: ['sslmode'],
+          visible: function(m) {
+            var sslrootcert = m.get('sslrootcert');
+            return !_.isUndefined(sslrootcert) && !_.isNull(sslrootcert);
+          }
+        },{
+          id: 'sslcrl', label: gettext('Certificate revocation list'), type: 'text',
+          group: gettext('SSL'), mode: ['properties'],
+          deps: ['sslmode'],
+          visible: function(m) {
+            var sslcrl = m.get('sslcrl');
+            return !_.isUndefined(sslcrl) && !_.isNull(sslcrl);
+          }
+        },{
+          id: 'sslcompression', label: gettext('SSL compression?'), type: 'switch',
+          mode: ['properties'], group: gettext('SSL'),
+          'options': { 'onText':  gettext('True'), 'offText':  gettext('False'),
+          'onColor':  'success', 'offColor': 'danger', 'size': 'small'},
+          deps: ['sslmode'], visible: function(m) {
+            var sslmode = m.get('sslmode');
+            return _.indexOf(SSL_MODES, sslmode) != -1;
+          }
+        },{
+          id: 'hostaddr', label: gettext('Host address'), type: 'text', group: gettext('Advanced'),
+          mode: ['properties', 'edit', 'create'], disabled: 'isConnected'
+        },{
+          id: 'db_res', label: gettext('DB restriction'), type: 'select2', group: gettext('Advanced'),
+          mode: ['properties', 'edit', 'create'], disabled: 'isConnected', select2: {multiple: true, allowClear: false,
+          tags: true, tokenSeparators: [','], first_empty: false, selectOnClose: true, emptyOptions: true}
+        },{
+          id: 'passfile', label: gettext('Password file'), type: 'text',
+          group: gettext('Advanced'), mode: ['edit', 'create'],
+          disabled: 'isConnectedWithValidLib', control: Backform.FileControl,
+          dialog_type: 'select_file', supp_types: ['*']
+        },{
+          id: 'passfile', label: gettext('Password file'), type: 'text',
+          group: gettext('Advanced'), mode: ['properties'],
+          visible: function(m) {
+            var passfile = m.get('passfile');
+            return !_.isUndefined(passfile) && !_.isNull(passfile);
+          }
         }],
         validate: function() {
           var err = {},
@@ -799,6 +925,39 @@ define('pgadmin.node.server', [
         },
         isConnected: function(model) {
           return model.get('connected');
+        },
+        isfgColorSet: function(model) {
+          var bgcolor = model.get('bgcolor'),
+              fgcolor = model.get('fgcolor');
+
+          if(model.get('connected')) {
+            return true;
+          }
+          // If fgcolor is set and bgcolor is not set then force bgcolor
+          // to set as white
+          if(_.isUndefined(bgcolor) || _.isNull(bgcolor) || !bgcolor) {
+            if(fgcolor) {
+              model.set('bgcolor', '#ffffff');
+            }
+          }
+
+          return false;
+        },
+        isSSL: function(model) {
+          var ssl_mode = model.get('sslmode');
+          // If server is not connected and have required SSL option
+          if(model.get('connected')) {
+            return true;
+          }
+          return _.indexOf(SSL_MODES, ssl_mode) == -1;
+        },
+        isConnectedWithValidLib: function(model) {
+          if(model.get('connected')) {
+            return true;
+          }
+          // older version of libpq do not support 'passfile' parameter in
+          // connect method, valid libpq must have version >= 100000
+          return pgBrowser.utils.pg_libpq_version < 100000;
         }
       }),
       connection_lost: function(i, resp) {
@@ -903,8 +1062,7 @@ define('pgadmin.node.server', [
                 pgBrowser.serverInfo || {};
               serverInfo[data._id] = _.extend({}, data);
 
-              var alertifyWrapper = new AlertifyWrapper();
-              alertifyWrapper.success(res.info);
+              alertify.success(res.info);
               obj.trigger('connected', obj, item, data);
 
               // Generate the event that server is connected
@@ -1037,7 +1195,7 @@ define('pgadmin.node.server', [
       };
 
       data.is_connecting = true;
-      url = obj.generate_url(item, "connect", data, true);
+      var url = obj.generate_url(item, "connect", data, true);
       $.post(url)
       .done(function(res) {
         if (res.success == 1) {
